/**********************************************************************
   Mark Butcher    Bsc (Hons) MPhil MIET

   M.J.Butcher Consulting
   Birchstrasse 20f,    CH-5406, Rtihof
   Switzerland

   www.uTasker.com    Skype: M_J_Butcher

   ---------------------------------------------------------------------
   File:        mass_storage.h
   Project:     uTasker Demonstration project
   ---------------------------------------------------------------------
   Copyright (C) M.J.Butcher Consulting 2004..2010
   *********************************************************************
   27.01.2010 Add UTDIR_SET_START flag                                   {1}
   31.01.2010 Add root_disk_location to UTDIRECTORY                      {2}
   31.01.2010 Add return values UTFAT_PATH_IS_ROOT, UTFAT_PATH_IS_ROOT_REF, UTFAT_SUCCESS_PATH_MODIFIED {3}
   31.01.2010 ucDirectoryFlags changed to wider usDirectoryFlags         {4}
   31.01.2010 Add directory flag UTDIR_ALLOW_MODIFY_PATH                 {5}
   31.01.2010 Extend utOpenFile() mode parameter from unsigned char to unsigned short {6}

*/
 


#if defined SDCARD_SUPPORT


#define SDCARDS                            1                             // this implementation supports one card with up to one partition

#define DISK_D                             0
#define DISK_E                             1
#define DISK_F                             2


#define WRITEBACK_BUFFER_FLAG              0x01
#define WRITEBACK_INFO_FLAG                0x02
#define FSINFO_VALID                       0x04
#define DISK_UNFORMATTED                   0x08
#define WRITE_PROTECTED_SD_CARD            0x10
#define DISK_FORMATTED                     0x20
#define HIGH_CAPACITY_SD_CARD              0x40
#define DISK_MOUNTED                       0x80

#define UTFAT_SUCCESS_PATH_MODIFIED        5                             // {3}
#define UTFAT_PATH_IS_ROOT                 4
#define UTFAT_PATH_IS_ROOT_REF             3
#define UTFAT_PATH_IS_FILE                 2
#define UTFAT_END_OF_DIRECTORY             1
#define UTFAT_SUCCESS                      0
#define UTFAT_DIR_NOT_EMPTY                -1
#define UTFAT_DISK_NOT_READY               -2
#define UTFAT_PATH_NOT_FOUND               -3
#define UTFAT_DISK_READ_ERROR              -4
#define UTFAT_DISK_WRITE_ERROR             -5
#define UTFAT_FILE_NOT_FOUND               -6
#define UTFAT_DIRECTORY_AREA_EXHAUSTED     -7
#define UTFAT_FILE_NOT_READABLE            -8
#define UTFAT_FILE_NOT_WRITEABLE           -9
#define UTFAT_FILE_LOCKED                  -10
#define UTFAT_SEARCH_INVALID               -11
#define UTFAT_DISK_ALREADY_FORMATTED       -12
#define UTFAT_NO_MORE_LISING_SPACE         -13
#define UTFAT_NO_MORE_LISTING_ITEMS_FOUND  -14
#define UTFAT_FINAL_LISTING_ITEM_FOUND     -15
#define UTFAT_DIRECTORY_OBJECT_MISSING     -16
#define UTFAT_DIRECTORY_EXISTS_ALREADY     -17


#define UTDIR_ALLOCATED                    0x0001                        // directory object is allocted to an application
#define UTDIR_VALID                        0x0002                        // the directory object is valid and can be used
#define UTDIR_REFERENCED                   0x0004                        // the access is referenced to the present directory path location
#define UTDIR_SET_START                    0x0008                        // {1} if the file is not found the directory is set to the lower directory in the path so that new files can be added there
#define UTDIR_DIR_AS_FILE                  0x0010                        // handle directories and files equivalently - used when renaming and deleting
#define UTDIR_TEST_REL_PATH                0x0020                        // test a path relative to the present directory path location but don't move to it
#define UTDIR_TEST_FULL_PATH               0x0040                        // temporary reference from the root directory
#define UTDIR_TEST_FULL_PATH_TEMP          0x0080                        // temporary reference from the present directory path reference
#define UTDIR_ALLOW_MODIFY_PATH            0x0100                        // {5} allow the directory search to modify the directory path string if it exists

#define BAD_CLUSTER_VALUE                  0x0ffffff7
#define CLUSTER_MASK                       0x0fffffff
#define MEDIA_VALUE_FIXED                  0x0ffffff8
#define MEDIA_VALUE_REMOVABLE              0x0ffffff0

#define FIXED_MEDIA                        0xf8
#define REMOVABLE_MEDIA                    0xf0


typedef struct stUTFAT
{
    unsigned long  ulFatSize;                                            // sectors occupied by ONE FAT
    unsigned long  ulFAT_start;                                          // the sector in which the first FAT starts (further FAT copies start at thsi sector + FAt size)
    unsigned long  ulClusterCount;                                       // total number of clusters in the cluster area
    unsigned short usBytesPerSector;                                     // the number of bytes in a single sector (512, 1024, 2048 or 4096)
    unsigned char  ucNumberOfFATs;                                       // the number of FATs (more that 1 are copies)
    unsigned char  ucSectorsPerCluster;                                  // the number of sectors in a singel cluster - cluster size is sector size x sectors per cluster
} UTFAT;



typedef struct stFILEINFO
{
    unsigned long ulInfoSector;                                          // sector where the information is located
    unsigned long ulFreeClusterCount;
    unsigned long ulNextFreeCluster;
    unsigned char ucCardSpecificData[16];                                // CSD register content
} FILEINFO;

typedef struct stUTDISK
{
    unsigned long  ulPresentSector;                                      // the present sector being used by the disk
    unsigned long  ulDirectoryBase;                                      // the first cluster in the root directory (usually 2)
    unsigned long  ulLogicalBaseAddress;                                 // first cluster containing data
    unsigned long  ulVirtualBaseAddress;                                 // virtual cluster startibg address, compensating unused clusters
    unsigned long  ulSD_sectors;                                         // physical sectors on the device
    unsigned char  *ptrSectorData;                                       // pointer to a buffer containing a copy of the sector data
    UTFAT          utFAT;                                                // FAT information concerning the data content
    FILEINFO       utFileInfo;                                           // file information used by FAT32
    unsigned char  ucDriveNumber;                                        // the drive number of this disk
    unsigned char  ucDiskFlags;                                          // flags indicating the status of the disk
    CHAR           cVolumeLabel[11];                                     // the volume's label
} UTDISK;


typedef struct stBOOT_SECTOR_BPB                                         // boot sector and bios parameter block
{
    unsigned char BS_jmpBoot[3];                                         // jump instruction to boot code
    CHAR          BS_OEMName[8];                                         // string usually indicating the system that formatted the volume - "MSWIN4.1" is recommended although MSDOS5.0 is typical
    unsigned char BPB_BytesPerSec[2];                                    // count of bytes per sector. This value may take on only the following values: 512, 1024, 2048 or 4096
    unsigned char BPB_SecPerClus;                                        // number of sectors per allocation unit. The legal values are 1, 2, 4, 8, 16, 32, 64, and 128 - however never cause a "bytes per cluster" value (BPB_BytesPerSec * BPB_SecPerClus) greater than 32K!!
    unsigned char BPB_RsvdSecCnt[2];                                     // number of reserved sectors in the reserved region of the volume starting at the first sector of the volume. Never 0. FAT12/16 always 1. FAT32 uses typcially 32
    unsigned char BPB_NumFATs;                                           // the count of FAT data structures on the volume. Recommended to be always 2 (although FLASH could use 1)
    unsigned char BPB_RootEntCnt[2];                                     // FAT12 and FAT16 volumes count of 32-byte directory entries in the root directory. FAT32 must always be 0. FAT16 should use 512. When multiplied by 32 it should result in an even multiple of BPB_BytesPerSec (for FAT12 and FAT16)
    unsigned char BPB_TotSec16[2];                                       // old 16-bit total count of sectors on the volume (in all four regions of the volume). May be zero is BPB_TotSec32  is non-zero. Must be 0 for FAT32
    unsigned char BPB_Media;                                             // 0xF0, 0xF8, 0xF9, 0xFA, 0xFB, 0xFC, 0xFD, 0xFE, or 0xFF. 0xF8 usually used for fixed and 0xF0 for removable media. Should match with FAT[0] entry but is otherwise obsolete
    unsigned char BPB_FATSz16[2];                                        // FAT12/FAT16 16-bit count of sectors occupied by ONE FAT. Must be 0 for FAT32
    unsigned char BPB_SecPerTrk[2];                                      // sectors per track for interrupt 0x13. Only valid for media whose volume is broken down into tracks by multiple heads and cylinders
    unsigned char BPB_NumHeads[2];                                       // number of heads for interrupt 0x13.
    unsigned char BPB_HiddSec[4];                                        // count of hidden sectors preceding the partition that contains this FAT volume. Should always be zero on media that are not partitioned but otherwise operating system dependent
    unsigned char BPB_TotSec32[4];                                       // 32-bit total count of sectors on the volume (all sectors in all four regions). Can be zero if BPB_TotSec16 is non-zero. Must be non-zero for FAT32
} BOOT_SECTOR_BPB;


typedef struct _PACK stBOOT_SECTOR_FAT12_FAT16
{
    BOOT_SECTOR_BPB boot_sector_bpb;                                     // standard boot sector and bios parameter block up to offset 36
    unsigned char BS_DrvNum;                                             // int 0x13 drive number - operating system specific
    unsigned char BS_Reserved1;                                          // should always be set to zero when formatting (is in fact used by Windows NT)
    unsigned char BS_BootSig;                                            // extended boot signature (0x29). Signature indicating that the following three fields are present
    unsigned char BS_VolID[4];                                           // volume serial number. Usually generated by simply combining the current date and time into a 32-bit value
    CHAR          BS_VolLab[11];                                         // label matching the 11-byte volume label recorded in the root directory. "NO NAME    " when no specific label
    CHAR          BS_FilSysType[8];                                      // "FAT12   ", "FAT16   ", or "FAT     ". Not actually used to determine type (more informational) and not used at all by Microsoft FAT
    unsigned char ucSpace[448];
    unsigned char ucCheck55;                                             // this location must be 0x55 - offset 510
    unsigned char ucCheckAA;                                             // this location must be 0xaa - offset 511
} BOOT_SECTOR_FAT12_FAT16;

typedef struct _PACK stBOOT_SECTOR_FAT32
{
    BOOT_SECTOR_BPB boot_sector_bpb;                                     // standard boot sector and bios parameter block up to offset 36
    unsigned char BPB_FATSz32[4];                                        // FAT32 32-bit count of sectors occupied by ONE FAT - BPB_FATSz16 must be zero!
    unsigned char BPB_ExtFlags[2];
    unsigned char BPB_FSVer[2];                                          // version number of the FAT32 volume. major:minor - 0:0 expected at the time of writing but could change in the future indicating changes
    unsigned char BPB_RootClus[4];                                       // cluster number of the first cluster of the root directory. Usually 2
    unsigned char BPB_FSInfo[2];                                         // sector number of FSINFO structure in the reserved area of the FAT32 volume. Usually 1
    unsigned char BPB_BkBootSec[2];                                      // sector number in the reserved area of the volume of a copy of the boot record (if non-zero). 6 is recommended
    unsigned char BPB_Reserved[12];                                      // should be 0
    unsigned char BS_DrvNum;                                             // int 0x13 drive number - operating system specific
    unsigned char BS_Reserved1;                                          // should always be set to zero when formatting (is in fact used by Windows NT)
    unsigned char BS_BootSig;                                            // extended boot signature (0x29). Signature indicating that the following three fields are present
    unsigned char BS_VolID[4];                                           // volume serial number. Usually generated by simply combining the current date and time into a 32-bit value
    CHAR          BS_VolLab[11];                                         // label matching the 11-byte volume label recorded in the root directory. "NO NAME    " when no specific label
    CHAR          BS_FilSysType[8];                                      // always "FAT32   ". Not actually used to determine type (more informational) and not used at all by Microsoft FAT
    unsigned char ucSpace[420];
    unsigned char ucCheck55;                                             // this location must be 0x55 - offset 510
    unsigned char ucCheckAA;                                             // this location must be 0xaa - offset 511
} BOOT_SECTOR_FAT32;


// FAT32 BPB_ExtFlags flags
//
#define BPB_ExtFlags_0_ZERO_BASED_NUMBER 0x0f                            // only valid if mirroring is disabled
#define BPB_ExtFlags_0_MIRRORED_FAT      0x00
#define BPB_ExtFlags_0_ONE_FAT           0x80                            // mirroring disabled


typedef struct stINFO_SECTOR_FAT32
{
    unsigned char FSI_LeadSig[4];                                        // value 0x41615252. Used to validate that this is in fact an FSInfo sector
    unsigned char FSI_Reserved1[480];                                    // content should be zero
    unsigned char FSI_StrucSig[4];                                       // value 0x61417272. More localized signature
    unsigned char FSI_Free_Count[4];                                     // last known free cluster count on the volume - 0xffffffff means unknown
    unsigned char FSI_Nxt_Free[4];                                       // indicates the cluster number at which the driver should start looking for free clusters - 0xffffffff means unknown
    unsigned char FSI_Reserved2[12];                                     // content should be zero
    unsigned char FSI_TrailSig[4];                                       // value 0xaa550000
} INFO_SECTOR_FAT32;


typedef struct stPARTITION_TABLE_ENTRY
{
    unsigned char boot_indicator;                                        // 0x80 indicates bootable
    unsigned char starting_cylinder;                                     // cylinder start value
    unsigned char starting_head;                                         // head start value
    unsigned char starting_sector;                                       // sector start value
    unsigned char partition_type;                                        // partition type descriptor
    unsigned char ending_cylinder;                                       // cylinder start value
    unsigned char ending_head;                                           // head start value
    unsigned char ending_sector;                                         // sector start value
    unsigned char start_sector[4];                                       // start sector
    unsigned char partition_size[4];                                     // partition size in sectors
} PARTITION_TABLE_ENTRY;


typedef struct _PACK stEXTENDED_BOOT_RECORD
{
    unsigned char         EBR_unused1[394];                              // generally 0
    unsigned char         EBR_IBM_menu[9];                               // possible IBM boot manager menu entry
    unsigned char         EBR_unused2[43];                               // generally 0
    PARTITION_TABLE_ENTRY EBR_partition_table[2];                        // two partition tables
    unsigned char         EBR_unused3[32];                               // generally 0
    unsigned char         ucCheck55;                                     // this location must be 0x55 - offset 510
    unsigned char         ucCheckAA;                                     // this location must be 0xaa - offset 511
} EXTENDED_BOOT_RECORD;


typedef struct stDIR_ENTRY_STRUCTURE_FAT32
{
    unsigned char DIR_Name[11];                                          // directory short name. If the first byte is 0xe5 the directory entry is free. If it is 0x00 this and all following are free. If it is 0x05 it means that the actual file name begins with 0xe5 (makes Japanese character set possible). May not start with ' ' or lower (apart from special case for 0x05) and lower case characters are not allowed. The following characters are not allowed: "	0x22, 0x2A, 0x2B, 0x2C, 0x2E, 0x2F, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F, 0x5B, 0x5C, 0x5D, and 0x7C
    unsigned char DIR_Attr;                                              // file attributes
    unsigned char DIR_NTRes;                                             // reserved for Windows NT - should be 0
    unsigned char DIR_CrtTimeTenth;                                      // millisecond stamp at file creation time. Actually contains a count of tenths of a second 0..199
    unsigned char DIR_CrtTime[2];                                        // time file was created
    unsigned char DIR_CrtDate[2];                                        // data file was created
    unsigned char DIR_LstAccDate[2];                                     // last access date (read or write), set to same as DIR_WrtDate on write
    unsigned char DIR_FstClusHI[2];                                      // high word of this entry's first cluster number (always 0 for a FAT12 or FAT16 volume)
    unsigned char DIR_WrtTime[2];                                        // time of last write, whereby a file creation is considered as a write
    unsigned char DIR_WrtDate[2];                                        // date of last write, whereby a file creation is considered as a write
    unsigned char DIR_FstClusLO[2];                                      // low word of this entry's first cluster number
    unsigned char DIR_FileSize[4];                                       // file's size in bytes
} DIR_ENTRY_STRUCTURE_FAT32;



typedef struct stLFN_ENTRY_STRUCTURE_FAT32
{
    unsigned char LFN_EntryNumber;                                       // entry number starting from last - 0x40 is always set and the value decrements until 1
    unsigned char LFN_Name_0;                                            // first letter
    unsigned char LFN_Name_0_extension;                                  // first letter extension - is always 0 in english character set
    unsigned char LFN_Name_1;                                            // second letter
    unsigned char LFN_Name_1_extension;                                  // second letter extension - is always 0 in english character set
    unsigned char LFN_Name_2;                                            // third letter
    unsigned char LFN_Name_2_extension;                                  // third letter extension - is always 0 in english character set
    unsigned char LFN_Name_3;                                            // fourth letter
    unsigned char LFN_Name_3_extension;                                  // fourth letter extension - is always 0 in english character set
    unsigned char LFN_Name_4;                                            // fifth letter
    unsigned char LFN_Name_4_extension;                                  // fifth letter extension - is always 0 in english character set
    unsigned char LFN_Attribute;                                         // always 0x0f
    unsigned char LFN_Zero0;                                             // always zero
    unsigned char LFN_Checksum;                                          // check sum
    unsigned char LFN_Name_5;                                            // sixth letter
    unsigned char LFN_Name_5_extension;                                  // sixth letter extension - is always 0 in english character set
    unsigned char LFN_Name_6;                                            // seventh letter
    unsigned char LFN_Name_6_extension;                                  // seventh letter extension - is always 0 in english character set
    unsigned char LFN_Name_7;                                            // eighth letter
    unsigned char LFN_Name_7_extension;                                  // eighth letter extension - is always 0 in english character set
    unsigned char LFN_Name_8;                                            // ninth letter
    unsigned char LFN_Name_8_extension;                                  // ninth letter extension - is always 0 in english character set
    unsigned char LFN_Name_9;                                            // tenth letter
    unsigned char LFN_Name_9_extension;                                  // tenth letter extension - is always 0 in english character set
    unsigned char LFN_Name_10;                                           // eleventh letter
    unsigned char LFN_Name_10_extension;                                 // eleventh letter extension - is always 0 in english character set
    unsigned char LFN_Zero1;                                             // always zero
    unsigned char LFN_Zero2;                                             // always zero
    unsigned char LFN_Name_11;                                           // twelfth letter
    unsigned char LFN_Name_11_extension;                                 // twelfth letter extension - is always 0 in english character set
    unsigned char LFN_Name_12;                                           // thirteenth letter
    unsigned char LFN_Name_12_extension;                                 // thirteenth letter extension - is always 0 in english character set
} LFN_ENTRY_STRUCTURE_FAT32;



#ifndef UTFAT_LFN_READ
    #undef MAX_UTFAT_FILE_NAME
    #define MAX_UTFAT_FILE_NAME (8 + 1 + 3)                              // 8.3 format
#endif

#define DOS_STYLE_LIST_ENTRY_LENGTH (43)
#define FTP_STYLE_LIST_ENTRY_LENGTH (46)


typedef struct stUTFILEINFO {
    unsigned long  ulFileSize;
    unsigned short usFileData;
    unsigned short usFileTime;
    unsigned char  ucFileAttributes;
    CHAR           cFileName[MAX_UTFAT_FILE_NAME + 1];
} UTFILEINFO;


typedef struct stFILE_LOCATION
{
    unsigned long ulCluster;
    unsigned long ulSector;
} FILE_LOCATION;

typedef struct stDISK_LOCATION
{
    FILE_LOCATION directory_location;
    unsigned char ucDirectoryEntry;
} DISK_LOCATION;


// Main directory object
//
typedef struct stUTDIRECTORY
{
    CHAR *                     ptrDirectoryPath;                         // pointer to the location where the directory path string is stored
    DIR_ENTRY_STRUCTURE_FAT32 *ptrEntryStructure;                        // pointer to a loaded entry
    DISK_LOCATION              root_disk_location;                       // {2} reference to the root directory
    DISK_LOCATION              private_disk_location;                    // details of where the directory is physically located on the disk
    DISK_LOCATION              public_disk_location;                     // working information when searching for and manipulating directory contents
    FILE_LOCATION              public_file_location;                     // reference to a file location
    unsigned short             usDirectoryPathLength;                    // the length of string reserved for use by the directory path
    unsigned short             usRelativePathLocation;                   // the present relative path location
    unsigned short             usDirectoryFlags;                         // {4} flags reflecting the directory's state and for defining specific operation
    unsigned char              ucDrive;                                  // the physical drive associated with the directory
} UTDIRECTORY;


// Directory list object, which works together with a main directory object
//
typedef struct stUTLISTDIRECTORY
{
    UTDIRECTORY   *ptr_utDirObject;                                      // pointer to the main directory object
    DISK_LOCATION private_disk_location;                                 // disk location which doesn't change when used
    unsigned long ulDirectorySector;                                     // the sector in which the directory entries are contained
    unsigned long ulDirectorySectorIncrement;                            // incremental sector value
    DIR_ENTRY_STRUCTURE_FAT32 *ptrEntryStructure;                        // pointer to a loaded entry
    unsigned char ucDirectoryEntry;                                      // directory entry in present sector (0..31) for 512 sized sectors, up to 255 for maximum 4k sectors
    unsigned char ucDirectoryEntryIncrement;                             // incremental directory entry in present sector
} UTLISTDIRECTORY;


// File access object, which works together with a main directory object
//
typedef struct stUTFILE 
{
    UTDIRECTORY    *ptr_utDirObject;                                     // pointer to the main directory object
    unsigned long  ulFileSize;                                           // file's total length
    unsigned long  ulFilePosition;                                       // present linear file position
    int            iFileHandle;                                          // file handle for use in managed mode
    DISK_LOCATION  private_disk_location;                                // location of the file information
    FILE_LOCATION  private_file_location;                                // location of the start of file data
    FILE_LOCATION  public_file_location;                                 // present file data location
    unsigned short usLastReadWriteLength;                                // read or write length last performed
    unsigned char  ucFileMode;                                           // mode that the file was opened for
    unsigned char  ucDrive;                                              // disk drive that the file is on
    UTASK_TASK     ownerTask;                                            // owner task for use by managed mode
} UTFILE;



typedef struct stFILE_LISTING
{
    CHAR *ptrBuffer;
    unsigned long  ulFileSizes;
    unsigned short usBufferLength;
    unsigned short usStringLength;
    unsigned short usMaxItems;
    unsigned short usItemsReturned;
    unsigned short usDirectoryCount;
    unsigned short usFileCount;
    unsigned char  ucStyle;
} FILE_LISTING;

#define DOS_TYPE_LISTING       0
#define FTP_TYPE_LISTING       1


// Parameters used when formatting a disk (values recommended by Microsoft and suiting the Microsoft dimensioning algorithm)
//
#define BOOT_SECTOR_LOCATION   63
#define BACKUP_ROOT_SECTOR     6
#define BYTES_PER_SECTOR       512
#define RESERVED_SECTION_COUNT 32
#define NUMBER_OF_FATS         2



extern UTDIRECTORY*utAllocateDirectory(unsigned char ucDisk, unsigned short ulPathLength);
extern UTDISK *fnGetDiskInfo(unsigned char ucDisk);
extern int  utOpenDirectory(const CHAR *ptrDirPath, UTDIRECTORY *ptrDirObject);
extern int  utLocateDirectory(const CHAR *ptrDirPath, UTLISTDIRECTORY *ptrListDirectory);
extern int  utMakeDirectory(const CHAR *ptrDirPath, UTDIRECTORY *ptrDirObject);
extern int  utChangeDirectory(const CHAR *ptrDirPath, UTDIRECTORY *ptrDirObject);
extern int  utDeleteFile(const CHAR *ptrFilePath, UTDIRECTORY *ptrDirObject);
extern int  utListDir(UTLISTDIRECTORY *ptr_utDirectory, FILE_LISTING *ptrFileLists);
extern int  utReadFile(UTFILE *ptr_utFile, void *ptrBuffer, unsigned short usReadLength);
extern int  utOpenFile(const CHAR *ptrFilePath, UTFILE *ptr_utFile, unsigned short usAccessMode); // {6}
extern int  utRenameFile(const CHAR *ptrFilePath, UTFILE *ptr_utFile);
extern void utCloseFile(UTFILE *ptr_utFile);
extern int  utWriteFile(UTFILE *ptr_utFile, unsigned char *ptrBuffer, unsigned short usLength);
extern int  utFormat(const unsigned char ucDrive, const CHAR *cVolumeLabel);
extern int  utReFormat(const unsigned char ucDrive, const CHAR *cVolumeLabel);
extern int  utSeek(UTFILE *ptr_utFile, unsigned long ulPosition, int iSeekType);
extern int  fnReadSector(unsigned char ucDisk, unsigned char *ptrBuffer, unsigned long ulSectorNumber);
extern int  utFreeClusters(unsigned char ucDisk, UTASK_TASK owner_task);

extern int  uOpenManagedFile(CHAR *ptrFileName, UTASK_TASK owner_task, unsigned char ucMode);

// Open modes - only the first byte is saved to the file object; the others are attributes for the controlling function
//
#define UTFAT_OPEN_FOR_READ    0x0001                                    // file to be opened for reading
#define UTFAT_OPEN_FOR_WRITE   0x0002                                    // file to be opened for writing to
#define UTFAT_OPEN_FOR_DELETE  0x0004                                    // file to be opened so that it can be deleted
#define UTFAT_PROTECTED        0x0008                                    // the file is to be opened and protected - no access by other users
#define UTFAT_MANAGED_MODE     0x0010                                    // open the file in managed mode so that any changes to it by other users are automatically updated
#define UTFAT_FILE_IS_DIR      0x0080                                    // the file is a directory type
#define UTFAT_OPEN_FOR_RENAME  0x0100                                    // open a file or directory in order to rename it
#define UTFAT_TRUNCATE         0x0200                                    // if the file already exists truncate it so that its length is zero
#define UTFAT_CREATE           0x0400                                    // if the file doesn't exist create it

#define UTFAT_SEEK_SET         0                                         // set the file position relative to the start of the file
#define UTFAT_SEEK_CUR         1                                         // set the file position relative to the current position
#define UTFAT_SEEK_END         2                                         // set the file position relative to the end fo the file

#endif
